@@ -1,9 +1,9 @@
 #!/bin/sh
 
-ZVER=$(cat /sys/module/zfs/version | cut -f 1 -d '-')
+ZVER=$(cut -f 1 -d '-' /sys/module/zfs/version)
 KVER=$(uname -r)
 
-bpftrace \
+exec bpftrace \
 	--include "/usr/src/zfs-$ZVER/$KVER/zfs_config.h" \
 	-I "/usr/src/zfs-$ZVER/include" \
 	-I "/usr/src/zfs-$ZVER/include/spl" \
