@@ -442,9 +442,10 @@ dm_get_underlying_path(const char *dm_name)
 	else
 		dev_str = tmp;
 
-	size = asprintf(&tmp, "/sys/block/%s/slaves/", dev_str);
-	if (size == -1 || !tmp)
+	if ((size = asprintf(&tmp, "/sys/block/%s/slaves/", dev_str)) == -1) {
+		tmp = NULL;
 		goto end;
+	}
 
 	dp = opendir(tmp);
 	if (dp == NULL)
@@ -470,7 +471,9 @@ dm_get_underlying_path(const char *dm_name)
 			if (!enclosure_path)
 				continue;
 
-			size = asprintf(&path, "/dev/%s", ep->d_name);
+			if ((size = asprintf(
+			    &path, "/dev/%s", ep->d_name)) == -1)
+				path = NULL;
 			free(enclosure_path);
 			break;
 		}
@@ -482,13 +485,14 @@ end:
 	free(tmp);
 	free(realp);
 
-	if (!path) {
+	if (!path && first_path) {
 		/*
 		 * None of the underlying paths had a link back to their
 		 * enclosure devices.  Throw up out hands and return the first
 		 * underlying path.
 		 */
-		size = asprintf(&path, "/dev/%s", first_path);
+		if ((size = asprintf(&path, "/dev/%s", first_path)) == -1)
+			path = NULL;
 	}
 
 	free(first_path);
@@ -526,7 +530,7 @@ zfs_dev_is_whole_disk(const char *dev_name)
 	struct dk_gpt *label;
 	int fd;
 
-	if ((fd = open(dev_name, O_RDONLY | O_DIRECT)) < 0)
+	if ((fd = open(dev_name, O_RDONLY | O_DIRECT | O_CLOEXEC)) < 0)
 		return (B_FALSE);
 
 	if (efi_alloc_and_init(fd, EFI_NUMPAR, &label) != 0) {
