@@ -233,9 +233,7 @@ arc_lowmem(void *arg __unused, int howto __unused)
 	 * with ARC reclaim thread.
 	 */
 	if (curproc == pageproc)
-		arc_wait_for_eviction(to_free);
-	else
-		arc_wait_for_eviction(0);
+		arc_wait_for_eviction(to_free, B_FALSE);
 }
 
 void
@@ -261,3 +259,13 @@ arc_lowmem_fini(void)
 	}
 #endif
 }
+
+void
+arc_register_hotplug(void)
+{
+}
+
+void
+arc_unregister_hotplug(void)
+{
+}
