@@ -261,9 +261,6 @@ zfs_read(struct znode *zp, zfs_uio_t *uio, int ioflag, cred_t *cr)
 	}
 
 	ASSERT(zfs_uio_offset(uio) < zp->z_size);
-#if defined(__linux__)
-	ssize_t start_offset = zfs_uio_offset(uio);
-#endif
 	ssize_t n = MIN(zfs_uio_resid(uio), zp->z_size - zfs_uio_offset(uio));
 	ssize_t start_resid = n;
 
@@ -286,18 +283,6 @@ zfs_read(struct znode *zp, zfs_uio_t *uio, int ioflag, cred_t *cr)
 			/* convert checksum errors into IO errors */
 			if (error == ECKSUM)
 				error = SET_ERROR(EIO);
-
-#if defined(__linux__)
-			/*
-			 * if we actually read some bytes, bubbling EFAULT
-			 * up to become EAGAIN isn't what we want here...
-			 *
-			 * ...on Linux, at least. On FBSD, doing this breaks.
-			 */
-			if (error == EFAULT &&
-			    (zfs_uio_offset(uio) - start_offset) != 0)
-				error = 0;
-#endif
 			break;
 		}
 
@@ -689,7 +674,7 @@ zfs_write(znode_t *zp, zfs_uio_t *uio, int ioflag, cred_t *cr)
 		}
 	}
 
-	zfs_inode_update(zp);
+	zfs_znode_update_vfs(zp);
 	zfs_rangelock_exit(lr);
 
 	/*
